@@ -74,6 +74,7 @@ struct diskdump_data {
 	ulong	evictions;		/* total evictions done */
 	ulong	cached_reads;
 	ulong  *valid_pages;
+	int     max_sect_len;           /* highest bucket of valid_pages */
 	ulong   accesses;
 	ulong	snapshot_task;
 };
@@ -509,6 +510,27 @@ arm_kdump_header_adjust(int header_version)
 }
 #endif  /* __i386__ && (ARM || MIPS) */
 
+/*
+ * Read page descriptor.
+ */
+static int
+read_pd(int fd, off_t offset, page_desc_t *pd)
+{
+	const off_t failed = (off_t)-1;
+
+	if (FLAT_FORMAT()) {
+		if (!read_flattened_format(fd, offset, pd, sizeof(*pd)))
+			return READ_ERROR;
+	} else {
+		if (lseek(fd, offset, SEEK_SET) == failed)
+			return SEEK_ERROR;
+		if (read(fd, pd, sizeof(*pd)) != sizeof(*pd))
+			return READ_ERROR;
+	}
+
+	return 0;
+}
+
 static int 
 read_dump_header(char *file)
 {
@@ -591,9 +613,12 @@ restart:
 	else if (STRNEQ(header->utsname.machine, "arm") &&
 	    machine_type_mismatch(file, "ARM", NULL, 0))
 		goto err;
-	else if (STRNEQ(header->utsname.machine, "mips") &&
+	else if (STREQ(header->utsname.machine, "mips") &&
 	    machine_type_mismatch(file, "MIPS", NULL, 0))
 		goto err;
+	else if (STRNEQ(header->utsname.machine, "mips64") &&
+	    machine_type_mismatch(file, "MIPS64", NULL, 0))
+		goto err;
 	else if (STRNEQ(header->utsname.machine, "s390x") &&
 	    machine_type_mismatch(file, "S390X", NULL, 0))
 		goto err;
@@ -751,7 +776,7 @@ restart:
 
 	if (machine_type("ARM"))
 		dd->machine_type = EM_ARM;
-	else if (machine_type("MIPS"))
+	else if (machine_type("MIPS") || machine_type("MIPS64"))
 		dd->machine_type = EM_MIPS;
 	else if (machine_type("X86"))
 		dd->machine_type = EM_386;
@@ -874,6 +899,7 @@ restart:
 	}
 
 	dd->valid_pages = calloc(sizeof(ulong), max_sect_len + 1);
+	dd->max_sect_len = max_sect_len;
 	for (i = 1; i < max_sect_len + 1; i++) {
 		dd->valid_pages[i] = dd->valid_pages[i - 1];
 		for (j = 0; j < BITMAP_SECT_LEN; j++, pfn++)
@@ -975,6 +1001,9 @@ is_diskdump(char *file)
 #ifdef SNAPPY
 	dd->flags |= SNAPPY_SUPPORTED;
 #endif
+#ifdef ZSTD
+	dd->flags |= ZSTD_SUPPORTED;
+#endif
 
 	pc->read_vmcoreinfo = vmcoreinfo_read_string;
 
@@ -1098,6 +1127,9 @@ cache_page(physaddr_t paddr)
 	const int block_size = dd->block_size;
 	const off_t failed = (off_t)-1;
 	ulong retlen;
+#ifdef ZSTD
+	static ZSTD_DCtx *dctx = NULL;
+#endif
 
 	for (i = found = 0; i < DISKDUMP_CACHED_PAGES; i++) {
 		if (DISKDUMP_VALID_PAGE(dd->page_cache_hdr[i].pg_flags))
@@ -1125,15 +1157,9 @@ cache_page(physaddr_t paddr)
 			+ (off_t)(desc_pos - 1)*sizeof(page_desc_t);
 
 	/* read page descriptor */
-	if (FLAT_FORMAT()) {
-		if (!read_flattened_format(dd->dfd, seek_offset, &pd, sizeof(pd)))
-			return READ_ERROR;
-	} else {
-		if (lseek(dd->dfd, seek_offset, SEEK_SET) == failed)
-			return SEEK_ERROR;
-		if (read(dd->dfd, &pd, sizeof(pd)) != sizeof(pd))
-			return READ_ERROR;
-	}
+	ret = read_pd(dd->dfd, seek_offset, &pd);
+	if (ret)
+		return ret;
 
 	/* sanity check */
 	if (pd.size > block_size)
@@ -1143,10 +1169,9 @@ cache_page(physaddr_t paddr)
 	if (FLAT_FORMAT()) {
 		if (!read_flattened_format(dd->dfd, pd.offset, dd->compressed_page, pd.size))
 			return READ_ERROR;
-	} else if (is_incomplete_dump() && (0 == pd.offset)) {
+	} else if (0 == pd.offset) {
 		/*
-		 *  If the incomplete flag has been set in the header, 
-		 *  first check whether zero_excluded has been set.
+		 *  First check whether zero_excluded has been set.
 		 */
 		if (*diskdump_flags & ZERO_EXCLUDED) {
 			if (CRASHDEBUG(8))
@@ -1155,8 +1180,15 @@ cache_page(physaddr_t paddr)
 				    "paddr/pfn: %llx/%lx\n", 
 					(ulonglong)paddr, pfn);
 			memset(dd->compressed_page, 0, dd->block_size);
-		} else
-			return READ_ERROR;
+		} else {
+			if (CRASHDEBUG(8))
+				fprintf(fp,
+					"read_diskdump/cache_page: "
+					"descriptor with zero offset found at "
+					"paddr/pfn/pos: %llx/%lx/%lx\n",
+					(ulonglong)paddr, pfn, desc_pos);
+			return PAGE_INCOMPLETE;
+		}
 	} else {
 		if (lseek(dd->dfd, pd.offset, SEEK_SET) == failed)
 			return SEEK_ERROR;
@@ -1225,6 +1257,33 @@ cache_page(physaddr_t paddr)
 			      ret);
 			return READ_ERROR;
 		}
+#endif
+	} else if (pd.flags & DUMP_DH_COMPRESSED_ZSTD) {
+
+		if (!(dd->flags & ZSTD_SUPPORTED)) {
+			error(INFO, "%s: uncompess failed: no zstd compression support\n",
+				DISKDUMP_VALID() ? "diskdump" : "compressed kdump");
+			return READ_ERROR;
+		}
+#ifdef ZSTD
+		if (!dctx) {
+			dctx = ZSTD_createDCtx();
+			if (!dctx) {
+				error(INFO, "%s: uncompess failed: cannot create ZSTD_DCtx\n",
+					DISKDUMP_VALID() ? "diskdump" : "compressed kdump");
+				return READ_ERROR;
+			}
+		}
+
+		retlen = ZSTD_decompressDCtx(dctx,
+				dd->page_cache_hdr[i].pg_bufptr, block_size,
+				dd->compressed_page, pd.size);
+		if (ZSTD_isError(retlen) || (retlen != block_size)) {
+			error(INFO, "%s: uncompress failed: %d (%s)\n",
+				DISKDUMP_VALID() ? "diskdump" : "compressed kdump",
+				retlen, ZSTD_getErrorName(retlen));
+			return READ_ERROR;
+		}
 #endif
 	} else
 		memcpy(dd->page_cache_hdr[i].pg_bufptr,
@@ -1466,6 +1525,12 @@ get_diskdump_regs_arm64(struct bt_info *bt, ulong *eip, ulong *esp)
 	machdep->get_stack_frame(bt, eip, esp);
 }
 
+static void
+get_diskdump_regs_mips(struct bt_info *bt, ulong *eip, ulong *esp)
+{
+	machdep->get_stack_frame(bt, eip, esp);
+}
+
 static void
 get_diskdump_regs_sparc64(struct bt_info *bt, ulong *eip, ulong *esp)
 {
@@ -1505,7 +1570,7 @@ get_diskdump_regs(struct bt_info *bt, ulong *eip, ulong *esp)
 		break;
 
 	case EM_MIPS:
-		return get_diskdump_regs_32(bt, eip, esp);
+		return get_diskdump_regs_mips(bt, eip, esp);
 		break;
 
 	case EM_386:
@@ -1691,7 +1756,7 @@ dump_note_offsets(FILE *fp)
 			qemu = FALSE;
 			if (machine_type("X86_64") || machine_type("S390X") ||
 			    machine_type("ARM64") || machine_type("PPC64") ||
-			    machine_type("SPARC64")) {
+			    machine_type("SPARC64") || machine_type("MIPS64")) {
 				note64 = (void *)dd->notes_buf + tot;
 				len = sizeof(Elf64_Nhdr);
 				if (STRNEQ((char *)note64 + len, "QEMU"))
@@ -1774,6 +1839,8 @@ __diskdump_memory_dump(FILE *fp)
 		fprintf(fp, "%sLZO_SUPPORTED", others++ ? "|" : "");
 	if (dd->flags & SNAPPY_SUPPORTED)
 		fprintf(fp, "%sSNAPPY_SUPPORTED", others++ ? "|" : "");
+	if (dd->flags & ZSTD_SUPPORTED)
+		fprintf(fp, "%sZSTD_SUPPORTED", others++ ? "|" : "");
         fprintf(fp, ") %s\n", FLAT_FORMAT() ? "[FLAT]" : "");
         fprintf(fp, "               dfd: %d\n", dd->dfd);
         fprintf(fp, "               ofp: %lx\n", (ulong)dd->ofp);
@@ -1840,6 +1907,8 @@ __diskdump_memory_dump(FILE *fp)
 			fprintf(fp, "DUMP_DH_COMPRESSED_LZO");
 		if (dh->status & DUMP_DH_COMPRESSED_SNAPPY)
 			fprintf(fp, "DUMP_DH_COMPRESSED_SNAPPY");
+		if (dh->status & DUMP_DH_COMPRESSED_ZSTD)
+			fprintf(fp, "DUMP_DH_COMPRESSED_ZSTD");
 		if (dh->status & DUMP_DH_COMPRESSED_INCOMPLETE)
 			fprintf(fp, "DUMP_DH_COMPRESSED_INCOMPLETE");
 		if (dh->status & DUMP_DH_EXCLUDED_VMEMMAP)
@@ -2074,6 +2143,7 @@ __diskdump_memory_dump(FILE *fp)
 	else
 		fprintf(fp, "\n");
 	fprintf(fp, "       valid_pages: %lx\n", (ulong)dd->valid_pages);
+	fprintf(fp, " total_valid_pages: %ld\n", dd->valid_pages[dd->max_sect_len]);
 
 	return 0;
 }
@@ -2477,6 +2547,12 @@ diskdump_display_regs(int cpu, FILE *ofp)
 		    UINT(user_regs + OFFSET(user_regs_struct_eflags))
 		);
 	}
+
+	if (machine_type("MIPS"))
+		mips_display_regs_from_elf_notes(cpu, ofp);
+
+	if (machine_type("MIPS64"))
+		mips64_display_regs_from_elf_notes(cpu, ofp);
 }
 
 void
@@ -2485,8 +2561,9 @@ dump_registers_for_compressed_kdump(void)
 	int c;
 
 	if (!KDUMP_CMPRS_VALID() || (dd->header->header_version < 4) ||
-	    !(machine_type("X86") || machine_type("X86_64") || 
-	      machine_type("ARM64") || machine_type("PPC64")))
+	    !(machine_type("X86") || machine_type("X86_64") ||
+	      machine_type("ARM64") || machine_type("PPC64") ||
+	      machine_type("MIPS") || machine_type("MIPS64")))
 		error(FATAL, "-r option not supported for this dumpfile\n");
 
 	if (machine_type("ARM64") && (kt->cpus != dd->num_prstatus_notes))
@@ -2516,13 +2593,22 @@ diskdump_kaslr_check()
 	return FALSE;
 }
 
-#ifdef X86_64
 int
 diskdump_get_nr_cpus(void)
 {
-	return dd->num_qemu_notes;
+        if (dd->num_prstatus_notes)
+                return dd->num_prstatus_notes;
+        else if (dd->num_qemu_notes)
+                return dd->num_qemu_notes;
+        else if (dd->num_vmcoredd_notes)
+                return dd->num_vmcoredd_notes;
+        else if (dd->header->nr_cpus)
+                return dd->header->nr_cpus;
+
+        return 1;
 }
 
+#ifdef X86_64
 QEMUCPUState *
 diskdump_get_qemucpustate(int cpu)
 {
@@ -2591,7 +2677,6 @@ diskdump_device_dump_info(FILE *ofp)
 	}
 }
 
-#ifdef LZO
 static void
 zram_init(void)
 {
@@ -2710,27 +2795,13 @@ lookup_swap_cache(ulonglong pte_val, unsigned char *zram_buf)
 	return NULL;
 }
 
-ulong (*decompressor)(unsigned char *in_addr, ulong in_size, unsigned char *out_addr, ulong *out_size, void *other/* NOT USED */);
-/*
- * If userspace address was swapped out to zram, this function is called to decompress the object.
- * try_zram_decompress returns decompressed page data and data length
- */
-ulong 
-try_zram_decompress(ulonglong pte_val, unsigned char *buf, ulong len, ulonglong vaddr)
+static int get_disk_name_private_data(ulonglong pte_val, ulonglong vaddr,
+				       char *name, ulong *private_data)
 {
-	char name[32] = {0};
-	ulonglong swp_offset;
-	ulong swap_info, bdev, bd_disk, zram, zram_table_entry, sector, index, entry, flags, size, outsize, off;
-	unsigned char *obj_addr = NULL;
-	unsigned char *zram_buf = NULL;
-	unsigned char *outbuf = NULL;
+	ulong swap_info, bdev, bd_disk;
 
-	if (INVALID_MEMBER(zram_compressor))
-		zram_init();
-
-	off = PAGEOFFSET(vaddr);
 	if (!symbol_exists("swap_info"))
-		return 0;
+		return FALSE;
 
 	swap_info = symbol_value("swap_info");
 
@@ -2747,87 +2818,143 @@ try_zram_decompress(ulonglong pte_val, unsigned char *buf, ulong len, ulonglong
 			sizeof(void *), "swap_info_struct_bdev", FAULT_ON_ERROR);
 	readmem(bdev + OFFSET(block_device_bd_disk), KVADDR, &bd_disk,
 			sizeof(void *), "block_device_bd_disk", FAULT_ON_ERROR);
-	readmem(bd_disk + OFFSET(gendisk_disk_name), KVADDR, name,
+	if (name)
+		readmem(bd_disk + OFFSET(gendisk_disk_name), KVADDR, name,
 			strlen("zram"), "gendisk_disk_name", FAULT_ON_ERROR);
-	if (!strncmp(name, "zram", strlen("zram"))) {
+	if (private_data)
+		readmem(bd_disk + OFFSET(gendisk_private_data), KVADDR,
+			private_data, sizeof(void *), "gendisk_private_data",
+			FAULT_ON_ERROR);
+
+	return TRUE;
+}
+
+ulong readswap(ulonglong pte_val, char *buf, ulong len, ulonglong vaddr)
+{
+	char name[32] = {0};
+
+	if (!get_disk_name_private_data(pte_val, vaddr, name, NULL))
+		return 0;
+
+	if (!strncmp(name, "zram", 4)) {
+		return try_zram_decompress(pte_val, (unsigned char *)buf, len, vaddr);
+	} else {
 		if (CRASHDEBUG(2))
-			error(WARNING, "this page has swapped to zram\n");
+			error(WARNING, "this page has been swapped to %s\n", name);
+		return 0;
+	}
+}
 
-		readmem(bd_disk + OFFSET(gendisk_private_data), KVADDR, &zram,
-				sizeof(void *), "gendisk_private_data", FAULT_ON_ERROR);
+ulong (*decompressor)(unsigned char *in_addr, ulong in_size, unsigned char *out_addr,
+			ulong *out_size, void *other/* NOT USED */);
+/*
+ * If userspace address was swapped out to zram, this function is called to decompress the object.
+ * try_zram_decompress returns decompressed page data and data length
+ */
+ulong
+try_zram_decompress(ulonglong pte_val, unsigned char *buf, ulong len, ulonglong vaddr)
+{
+	char name[32] = {0};
+	ulonglong swp_offset;
+	unsigned char *obj_addr = NULL;
+	unsigned char *zram_buf = NULL;
+	unsigned char *outbuf = NULL;
+	ulong zram, zram_table_entry, sector, index, entry, flags, size,
+		outsize, off;
 
-		readmem(zram + OFFSET(zram_compressor), KVADDR, name,
-			sizeof(name), "zram compressor", FAULT_ON_ERROR);
-		if (STREQ(name, "lzo")) {
-			if (!(dd->flags & LZO_SUPPORTED)) {
-				if (lzo_init() == LZO_E_OK)
-					dd->flags |= LZO_SUPPORTED;
-				else
-					return 0;
-			}
-			decompressor = (void *)lzo1x_decompress_safe;
-		} else {//todo,support more compressor
-			error(WARNING, "only the lzo compressor is supported\n");
+	if (INVALID_MEMBER(zram_compressor)) {
+		zram_init();
+		if (INVALID_MEMBER(zram_compressor)) {
+			error(WARNING,
+			      "Some pages are swapped out to zram. "
+			      "Please run mod -s zram.\n");
 			return 0;
 		}
+	}
 
-		if (THIS_KERNEL_VERSION >= LINUX(2, 6, 0)) {
-			swp_offset = (ulonglong)__swp_offset(pte_val);
-		} else {
-			swp_offset = (ulonglong)SWP_OFFSET(pte_val);
-		}
+	if (CRASHDEBUG(2))
+		error(WARNING, "this page has swapped to zram\n");
 
-		zram_buf = (unsigned char *)GETBUF(PAGESIZE());
-		/*lookup page from swap cache*/
-		obj_addr = lookup_swap_cache(pte_val, zram_buf);
-		if (obj_addr != NULL) {
-			memcpy(buf, obj_addr + off, len);
-			goto out;
-		}
-
-		sector = swp_offset << (PAGESHIFT() - 9);
-		index = sector >> SECTORS_PER_PAGE_SHIFT;
-		readmem(zram, KVADDR, &zram_table_entry,
-				sizeof(void *), "zram_table_entry", FAULT_ON_ERROR);
-		zram_table_entry += (index * SIZE(zram_table_entry));
-		readmem(zram_table_entry, KVADDR, &entry,
-				sizeof(void *), "entry of table", FAULT_ON_ERROR);
-		readmem(zram_table_entry + OFFSET(zram_table_flag), KVADDR, &flags,
-				sizeof(void *), "zram_table_flag", FAULT_ON_ERROR);
-		if (!entry || (flags & ZRAM_FLAG_SAME_BIT)) {
-			memset(buf, entry, len);
-			goto out;
-		}
-		size = flags & (ZRAM_FLAG_SHIFT -1);
-		if (size == 0) {
-			len = 0;
-			goto out;
-		}
-
-		readmem(zram + OFFSET(zram_mempoll), KVADDR, &zram,
-				sizeof(void *), "zram_mempoll", FAULT_ON_ERROR);
-
-		obj_addr = zram_object_addr(zram, entry, zram_buf);
-		if (obj_addr == NULL) {
-			len = 0;
-			goto out;
-		}
-
-		if (size == PAGESIZE()) {
-			memcpy(buf, obj_addr + off, len);
-		} else {
-			outbuf = (unsigned char *)GETBUF(PAGESIZE());
-			outsize = PAGESIZE();
-			if (!decompressor(obj_addr, size, outbuf, &outsize, NULL))
-				memcpy(buf, outbuf + off, len);
-			else {
-				error(WARNING, "zram decompress error\n");
-				len = 0;
-			}
-			FREEBUF(outbuf);
-		}
-	} else {
+	if (!get_disk_name_private_data(pte_val, vaddr, NULL, &zram))
 		return 0;
+
+	readmem(zram + OFFSET(zram_compressor), KVADDR, name,
+		sizeof(name), "zram compressor", FAULT_ON_ERROR);
+	if (STREQ(name, "lzo")) {
+#ifdef LZO
+		if (!(dd->flags & LZO_SUPPORTED)) {
+			if (lzo_init() == LZO_E_OK)
+				dd->flags |= LZO_SUPPORTED;
+			else
+				return 0;
+		}
+		decompressor = (void *)lzo1x_decompress_safe;
+#else
+		error(WARNING,
+		      "zram decompress error: this executable needs to be built"
+		      " with lzo library\n");
+		return 0;
+#endif
+	} else { /* todo: support more compressor */
+		error(WARNING, "only the lzo compressor is supported\n");
+		return 0;
+	}
+
+	if (THIS_KERNEL_VERSION >= LINUX(2, 6, 0)) {
+		swp_offset = (ulonglong)__swp_offset(pte_val);
+	} else {
+		swp_offset = (ulonglong)SWP_OFFSET(pte_val);
+	}
+
+	zram_buf = (unsigned char *)GETBUF(PAGESIZE());
+	/* lookup page from swap cache */
+	off = PAGEOFFSET(vaddr);
+	obj_addr = lookup_swap_cache(pte_val, zram_buf);
+	if (obj_addr != NULL) {
+		memcpy(buf, obj_addr + off, len);
+		goto out;
+	}
+
+	sector = swp_offset << (PAGESHIFT() - 9);
+	index = sector >> SECTORS_PER_PAGE_SHIFT;
+	readmem(zram, KVADDR, &zram_table_entry,
+		sizeof(void *), "zram_table_entry", FAULT_ON_ERROR);
+	zram_table_entry += (index * SIZE(zram_table_entry));
+	readmem(zram_table_entry, KVADDR, &entry,
+		sizeof(void *), "entry of table", FAULT_ON_ERROR);
+	readmem(zram_table_entry + OFFSET(zram_table_flag), KVADDR, &flags,
+		sizeof(void *), "zram_table_flag", FAULT_ON_ERROR);
+	if (!entry || (flags & ZRAM_FLAG_SAME_BIT)) {
+		memset(buf, entry, len);
+		goto out;
+	}
+	size = flags & (ZRAM_FLAG_SHIFT -1);
+	if (size == 0) {
+		len = 0;
+		goto out;
+	}
+
+	readmem(zram + OFFSET(zram_mempoll), KVADDR, &zram,
+		sizeof(void *), "zram_mempoll", FAULT_ON_ERROR);
+
+	obj_addr = zram_object_addr(zram, entry, zram_buf);
+	if (obj_addr == NULL) {
+		len = 0;
+		goto out;
+	}
+
+	if (size == PAGESIZE()) {
+		memcpy(buf, obj_addr + off, len);
+	} else {
+		outbuf = (unsigned char *)GETBUF(PAGESIZE());
+		outsize = PAGESIZE();
+		if (!decompressor(obj_addr, size, outbuf, &outsize, NULL))
+			memcpy(buf, outbuf + off, len);
+		else {
+			error(WARNING, "zram decompress error\n");
+			len = 0;
+		}
+		FREEBUF(outbuf);
 	}
 
 out:
@@ -2836,9 +2963,3 @@ out:
 	FREEBUF(zram_buf);
 	return len;
 }
-#else
-ulong try_zram_decompress(ulonglong pte_val, unsigned char *buf, ulong len, ulonglong vaddr)
-{
-	return 0;
-}
-#endif
