@@ -89,7 +89,8 @@ kvmdump_init(char *filename, FILE *fptr)
 	FILE *tmpfp;
 
 	if (!machine_type("X86") && !machine_type("X86_64")) {
-		error(FATAL, "invalid host architecture for KVM: %s\n",
+		error(FATAL, 
+		    "invalid or unsupported host architecture for KVM: %s\n",
 			MACHINE_TYPE);
 		return FALSE;
 	}
@@ -845,8 +846,10 @@ kvmdump_mapfile_exists(void)
 
 	sprintf(filename, "%s.map", pc->dumpfile);
 
-	if (!file_exists(filename, &stat) || !S_ISREG(stat.st_mode))
+	if (!file_exists(filename, &stat) || !S_ISREG(stat.st_mode)) {
+		free(filename);
 		return FALSE;
+	}
 
 	if (is_kvmdump_mapfile(filename)) {
 		pc->kvmdump_mapfile = filename;
